import unittest
from unittest import mock

import waldur_batch_allocation


@mock.patch("waldur_batch_allocation.waldur_client_from_module")
@mock.patch("waldur_batch_allocation.AnsibleModule")
class BatchAllocationMakingTest(unittest.TestCase):
    def setUp(self):
        module = mock.Mock()
        module.params = {
            "api_url": "http://example.com:8000/api",
            "access_token": "token",
            "offering": "Test offering",
            "plan": "Test plan",
            "project": "Test project",
            "cpu_hours": 100,
            "gpu_hours": 50,
            "ram_gb": 60,
            "name": "Alloc",
            "description": "Sample allocation",
        }
        module.check_mode = False
        self.module = module

    def test_create_batch_allocation(self, mock_ansible_module, mock_ansible_client):
        client = mock.Mock()
        client.create_marketplace_order.return_value = {"uuid": "order_uuid"}
        _, has_changed = waldur_batch_allocation.send_request_to_waldur(
            client, self.module
        )
        client.create_marketplace_order.assert_called_once()
        self.assertTrue(has_changed)



import unittest
from unittest import mock

import waldur_os_security_group

WEB = {
    "url": "api/123",
    "description": "descr",
    "rules": [
        {
            "from_port": "80",
            "to_port": "80",
            "cidr": "192.168.0.0/28",
            "protocol": "tcp",
            "direction": "ingress",
            "ethertype": "IPv4",
            "remote_group": None,
        }
    ],
}

SSH = {
    "url": "api/124",
    "description": "descr",
    "rules": [
        {
            "from_port": "80",
            "to_port": "80",
            "remote_group": "api/123",
            "protocol": "tcp",
            "direction": "ingress",
            "cidr": None,
            "ethertype": "IPv4",
        }
    ],
}


def fail_side_effect(*args, **kwargs):
    raise Exception(kwargs["msg"])


class SecurityGroupCreateTest(unittest.TestCase):
    def setUp(self) -> None:
        self.create_sg_call_kwargs = dict(
            project=None,
            tenant="tenant",
            name="sec-group",
            description="descr",
            rules=[],
            tags=None,
            wait=True,
            interval=20,
            timeout=600,
        )

        module = mock.Mock()
        module.params = {
            "access_token": "token",
            "api_url": "api",
            "tenant": "tenant",
            "description": "descr",
            "state": "present",
            "name": "sec-group",
            "wait": True,
            "interval": 20,
            "timeout": 600,
        }
        module.check_mode = False
        self.module = module

        self.client = mock.Mock()

    def check_successful_function_call(self):
        has_changed = waldur_os_security_group.send_request_to_waldur(
            self.client, self.module
        )

        self.client.create_security_group.assert_called_once_with(
            **self.create_sg_call_kwargs
        )
        self.assertTrue(has_changed)

    def check_unsuccessful_function_call(self, msg):
        self.module.fail_json.side_effect = fail_side_effect

        self.assertRaisesRegex(
            Exception,
            msg,
            waldur_os_security_group.send_request_to_waldur,
            self.client,
            self.module,
        )

    def test_group_creation_with_link_to_remote_group(self):
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "remote_group": "web",
                "protocol": "tcp",
            }
        ]

        self.client.get_security_group.side_effect = [WEB, None]

        self.create_sg_call_kwargs["rules"].append(
            {
                "from_port": "80",
                "to_port": "80",
                "remote_group": "api/123",
                "protocol": "tcp",
                "direction": "ingress",
            }
        )

        self.check_successful_function_call()

    def test_if_marketplace_resource_uuid_has_been_passed(self):
        self.client.get_marketplace_resource.return_value = {
            "scope": "http://exapmle.com/api/model/tenant/"
        }
        self.client._get.return_value = {"uuid": "tenant"}
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "remote_group": "web",
                "protocol": "tcp",
            }
        ]

        self.module.params.pop("tenant")
        self.module.params["waldur_resource"] = "waldur_resource_uuid"

        self.client.get_security_group.side_effect = [WEB, None]

        self.create_sg_call_kwargs["rules"].append(
            {
                "from_port": "80",
                "to_port": "80",
                "remote_group": "api/123",
                "protocol": "tcp",
                "direction": "ingress",
            }
        )

        self.check_successful_function_call()
        self.client.get_marketplace_resource.assert_called_once()

    def test_group_creation_erred_with_invalid_params(self):
        self.module.params["rules"] = [
            {"from_port": "80", "to_port": "80", "protocol": "tcp"}
        ]

        self.module.fail_json.side_effect = fail_side_effect

        self.check_unsuccessful_function_call(
            "Either cidr or remote_group must be specified."
        )

    def test_group_creation_with_valid_ipv4_cidr(self):
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "192.168.0.0/28",
                "protocol": "tcp",
            }
        ]

        self.client.get_security_group.return_value = None

        self.create_sg_call_kwargs["rules"].append(
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "192.168.0.0/28",
                "protocol": "tcp",
                "ethertype": "IPv4",
                "direction": "ingress",
            }
        )

        self.check_successful_function_call()

    def test_group_creation_with_valid_ipv6_cidr(self):
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "2002::/16",
                "protocol": "tcp",
                "ethertype": "IPv6",
            }
        ]

        self.client.get_security_group.return_value = None

        self.create_sg_call_kwargs["rules"].append(
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "2002::/16",
                "protocol": "tcp",
                "ethertype": "IPv6",
                "direction": "ingress",
            }
        )

        self.check_successful_function_call()

    def test_group_creation_with_invalid_v6_address(self):
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "192.168.0.0/28",
                "protocol": "tcp",
                "ethertype": "IPv6",
            }
        ]

        self.check_unsuccessful_function_call("Invalid IPv6 address.")

    def test_group_creation_with_invalid_v4_address(self):
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "2002::/16",
                "protocol": "tcp",
                "ethertype": "IPv4",
            }
        ]

        self.check_unsuccessful_function_call("Invalid IPv4 address.")

    def test_group_creation_with_invalid_ethertype(self):
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "2002::/16",
                "protocol": "tcp",
                "ethertype": "ABC",
            }
        ]

        self.check_unsuccessful_function_call("Invalid ethertype")

    def test_group_creation_with_ingress_direction(self):
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "192.168.0.0/28",
                "protocol": "tcp",
                "direction": "ingress",
            }
        ]

        self.client.get_security_group.return_value = None

        self.create_sg_call_kwargs["rules"].append(
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "192.168.0.0/28",
                "protocol": "tcp",
                "direction": "ingress",
                "ethertype": "IPv4",
            }
        )

        self.check_successful_function_call()

    def test_group_creation_with_egress_direction(self):
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "192.168.0.0/28",
                "protocol": "tcp",
                "direction": "egress",
            }
        ]

        self.client.get_security_group.return_value = None

        self.create_sg_call_kwargs["rules"].append(
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "192.168.0.0/28",
                "protocol": "tcp",
                "direction": "egress",
                "ethertype": "IPv4",
            }
        )

        self.check_successful_function_call()

    def test_group_creation_with_invalid_direction(self):
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "192.168.0.0/28",
                "protocol": "tcp",
                "direction": "invalid",
            }
        ]

        self.check_unsuccessful_function_call("Invalid direction .")

    def test_group_creation_if_group_already_exists(self):
        self.module.params["name"] = "ssh"
        self.module.params["rules"] = [
            {
                "from_port": "80",
                "to_port": "80",
                "remote_group": "web",
                "protocol": "tcp",
                "direction": "ingress",
            }
        ]

        self.client.get_security_group.side_effect = [WEB, SSH]

        has_changed = waldur_os_security_group.send_request_to_waldur(
            self.client, self.module
        )

        self.assertFalse(has_changed)

    def test_security_group_rules_comparison_with_cidr_positive(self):
        local_rules = [
            {
                "from_port": "80",
                "to_port": "80",
                "cidr": "192.168.0.0/28",
                "protocol": "tcp",
                "direction": "ingress",
                "ethertype": "IPv4",
            }
        ]

        remote_rules = WEB["rules"]

        self.assertTrue(
            waldur_os_security_group.compare_rules(local_rules, remote_rules)
        )

    def test_security_group_rules_with_remote_link_comparison_positive(self):
        local_rules = [
            {
                "from_port": "80",
                "to_port": "80",
                "remote_group": "api/123",
                "protocol": "tcp",
                "direction": "ingress",
                "ethertype": "IPv4",
            }
        ]

        remote_rules = SSH["rules"]

        self.assertFalse(
            waldur_os_security_group.compare_rules(local_rules, remote_rules)
        )

    def test_security_group_rules_comparison_with_cidr_negative(self):
        local_rules = [
            {
                "from_port": "81",
                "to_port": "81",
                "cidr": "192.168.0.0/28",
                "protocol": "tcp",
                "direction": "ingress",
                "ethertype": "IPv4",
            }
        ]

        remote_rules = WEB["rules"]

        self.assertFalse(
            waldur_os_security_group.compare_rules(local_rules, remote_rules)
        )

    def test_security_group_rules_with_remote_link_comparison_negative(self):
        local_rules = [
            {
                "from_port": "80",
                "to_port": "80",
                "remote_group": "api/124",
                "protocol": "tcp",
                "direction": "ingress",
                "ethertype": "IPv4",
            }
        ]

        remote_rules = SSH["rules"]

        self.assertFalse(
            waldur_os_security_group.compare_rules(local_rules, remote_rules)
        )


class CompareRulesFunctionTest(unittest.TestCase):
    def setUp(self) -> None:
        self.rules_1 = {
            "from_port": "80",
            "to_port": "80",
            "cidr": "192.168.0.0/28",
            "protocol": "tcp",
            "direction": "ingress",
            "ethertype": "IPv4",
        }
        self.rules_2 = {
            "from_port": "100",
            "to_port": "100",
            "cidr": "192.168.0.0/28",
            "protocol": "tcp",
            "direction": "ingress",
            "ethertype": "IPv4",
        }

    def test_compare_rules(self):
        self.assertTrue(
            waldur_os_security_group.compare_rules([self.rules_1], [self.rules_1])
        )

        self.assertFalse(
            waldur_os_security_group.compare_rules([self.rules_1], [self.rules_2])
        )

        self.assertTrue(
            waldur_os_security_group.compare_rules(
                [self.rules_1, self.rules_2], [self.rules_1, self.rules_2]
            )
        )

        self.assertTrue(
            waldur_os_security_group.compare_rules(
                [self.rules_1, self.rules_2], [self.rules_2, self.rules_1]
            )
        )

    def test_compare_rules_if_remote_group_passed(self):
        local_1 = {
            "from_port": "100",
            "to_port": "100",
            "cidr": "192.168.0.0/28",
            "protocol": "tcp",
            "direction": "ingress",
            "ethertype": "IPv4",
        }
        remote_1 = {
            "from_port": "100",
            "to_port": "100",
            "cidr": "192.168.0.0/28",
            "protocol": "tcp",
            "direction": "ingress",
            "ethertype": "IPv4",
            "remote_group": "api/124",
        }

        self.assertTrue(waldur_os_security_group.compare_rules([local_1], [remote_1]))

        local_2 = {
            "from_port": "80",
            "to_port": "80",
            "protocol": "tcp",
            "direction": "ingress",
            "remote_group": "api/124",
        }
        remote_2 = {
            "from_port": "80",
            "to_port": "80",
            "cidr": "192.168.0.0/28",
            "protocol": "tcp",
            "direction": "ingress",
            "ethertype": "IPv4",
            "remote_group": "api/124",
        }

        self.assertTrue(waldur_os_security_group.compare_rules([local_2], [remote_2]))
        self.assertTrue(
            waldur_os_security_group.compare_rules(
                [local_1, local_2], [remote_2, remote_1]
            )
        )

        remote_2.pop("remote_group")
        self.assertFalse(waldur_os_security_group.compare_rules([local_2], [remote_2]))


class CompareDescriptionFunctionTest(unittest.TestCase):
    def test_compare_description(self):
        self.assertTrue(
            waldur_os_security_group.compare_description("description", "description")
        )

        self.assertFalse(
            waldur_os_security_group.compare_description(
                "description", "new description"
            )
        )

        self.assertTrue(waldur_os_security_group.compare_description("", "  "))

        self.assertTrue(waldur_os_security_group.compare_description("", None))



#!/usr/bin/python
# has to be a full import due to Ansible 2.0 compatibility
from ansible.module_utils.basic import AnsibleModule
from waldur_client import (
    WaldurClientException,
    waldur_client_from_module,
    waldur_full_argument_spec,
)

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "OpenNode",
}

DOCUMENTATION = """
---
module: waldur_marketplace_os_get_instance
short_description: Get existing OpenStack instance
version_added: 0.1
description:
  - Get an OpenStack instance
requirements:
  - python = 3.8
  - requests
  - python-waldur-client
options:
  access_token:
    description:
      - An access token which has permissions to create an OpenStack instances.
    required: true
  api_url:
    description:
      - Fully qualified url to the Waldur.
    required: true
  name:
    description:
      - The name or UUID of existing OpenStack instance.
    required: true
  project:
    description:
      - The name or UUID of the project where instance is created.
    required: true
"""

EXAMPLES = """
- name: Get an OpenStack instance
  hosts: localhost
  tasks:
    - name: get instance
      waldur_marketplace_os_get_instance:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        name: Warehouse instance
        project: OpenStack Project
"""


def main():
    fields = {
        "name": {"required": True, "type": "str"},
        "project": {"required": False, "type": "str"},
    }
    module = AnsibleModule(argument_spec=waldur_full_argument_spec(**fields))

    client = waldur_client_from_module(module)
    try:
        instance = client.get_instance_via_marketplace(
            name=module.params["name"],
            project=module.params["project"],
        )
    except WaldurClientException as e:
        module.fail_json(msg=str(e))
    else:
        module.exit_json(instance=instance)


if __name__ == "__main__":
    main()



import unittest
from unittest import mock

import waldur_batch_offering


@mock.patch("waldur_batch_offering.waldur_client_from_module")
@mock.patch("waldur_batch_offering.AnsibleModule")
class CreateOfferingTest(unittest.TestCase):
    def setUp(self):
        module = mock.Mock()
        module.params = {
            "api_url": "http://example.com:8000/api",
            "access_token": "token",
            "name": "Test offering",
            "category": "Category UUID",
            "plans": ["test-plan-info"],
            "batch_service": "SLURM",
            "hostname": "localhost",
            "username": "user",
            "port": "8080",
            "gateway": "localhost",
            "default_account": "root",
        }
        module.check_mode = False
        self.module = module

    def test_create_offering(self, mock_ansible_module, mock_ansible_client):
        client = mock.Mock()
        client.create_offering.return_value = {
            "offering": [{"uuid": "offering_uuid"}],
            "changed": True,
        }
        offering, has_changed = waldur_batch_offering.send_request_to_waldur(
            client, self.module
        )
        client.create_offering.assert_called_once()
        self.assertTrue(has_changed)



#!/usr/bin/python
# has to be a full import due to Ansible 2.0 compatibility
import copy
from ipaddress import AddressValueError, IPv4Interface, IPv6Interface, NetmaskValueError

from ansible.errors import AnsibleError
from ansible.module_utils.basic import AnsibleModule
from waldur_client import (
    WaldurClientException,
    waldur_client_from_module,
    waldur_resource_argument_spec,
)

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "OpenNode",
}

DOCUMENTATION = """
---
module: waldur_os_security_group
short_description: Add/Update/Remove OpenStack tenant security group
version_added: 0.1
description:
  - "Add/Update/Remove OpenStack tenant security group"
requirements:
  - "python = 3.8"
  - "requests"
  - "python-waldur-client"
options:
  access_token:
    description:
      - An access token which has permissions to create a security group.
    required: true
  api_url:
    description:
      - Fully qualified URL to the Waldur.
    required: true
  description:
    description:
      - A description of the security group.
    required: false
  interval:
    default: 20
    description:
      - An interval of the security group state polling.
  name:
    description:
      - The name of the security group.
    required: true
  project:
    description:
      - Name or UUID of the Waldur project where OpenStack tenant is located.
    required: false
  rules:
    description:
      - A list of security group rules to be applied to the security group.
        A rule consists of 5 fields: 'to_port', 'from_port', 'protocol', 'ethertype',
        'direction' and either 'cidr' or 'remote_group' (remote group name).
        'ethertype' (IPv4 by default) and 'direction' (ingress by default)
        parameters are optional.
  state:
    choices:
      - present
      - absent
    default: present
    description:
      - Should the resource be present or absent.
  tags:
    description:
      - List of tags that will be added to the security group on provisioning.
  tenant:
    description:
      - The name of the tenant to create a security group for.
    required: true
  timeout:
    default: 600
    description:
      - The maximum amount of seconds to wait until the security group provisioning is finished.
  wait:
    default: true
    description:
      - A boolean value that defines whether client has to wait until the security group is provisioned.
"""

EXAMPLES = """
- name: add security group
  hosts: localhost
  tasks:
    - name: create security group
      waldur_os_security_group:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: VPC #1
        description: http and https ports group
        rules:
          - from_port: 80
            to_port: 80
            cidr: 0.0.0.0/0
            protocol: tcp
          - from_port: 443
            to_port: 443
            cidr: 0.0.0.0/0
            protocol: tcp
        state: present
        name: classic-web

- name: remove security group
  hosts: localhost
  tasks:
    - name: remove previous security group
      waldur_os_security_group:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        name: classic-web
        tenant: VPC #1
        state: absent

- name: add security group
  hosts: localhost
  tasks:
    - name: create security group with 1 security rule
      waldur_os_security_group:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: VPC #1
        description: http only
        rules:
          - from_port: 80
            to_port: 80
            cidr: 0.0.0.0/0
            protocol: tcp
        state: present
        name: classic-web
        tags:
            - ansible_application_id

- name: update rules of security group
  hosts: localhost
  tasks:
    - name: update rules of security group
      waldur_os_security_group:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: VPC #1
        state: present
        name: postgresql
        rules:
          - to_port: 22
            cidr: 0.0.0.0/0
            from_port: 22
            protocol: tcp

          - to_port: -1
            cidr: 0.0.0.0/0
            from_port: -1
            protocol: icmp

          - to_port: 5432
            cidr: 0.0.0.0/0
            from_port: 5432
            protocol: tcp

- name: add security group with empty rules
  hosts: localhost
  tasks:
    - name: create security group
      waldur_os_security_group:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: VPC
        description: empty group
        state: present
        name: empty

- name: add security group using remote group
  hosts: localhost
  tasks:
    - name: create security group with a link to remote group
      waldur_os_security_group:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: VPC
        description: group depending on a remote group
        rules:
        - from_port: 80
          to_port: 80
          remote_group: web
          protocol: tcp
        state: present
        name: group_remote_group

- name: add security group with cidr ethertype
  hosts: localhost
  gather_facts: no
  tasks:
    - name: create security group with cidr ethertype
      waldur_os_security_group:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: waldur-dev-infra
        description: some descr
        rules:
        - from_port: 80
          to_port: 80
          cidr: 0.0.0.0/00
          protocol: tcp
          ethertype: IPv4
        state: present
        name: sec-group-with-ethertype

- name: add security group with IPv6 cidr
  hosts: localhost
  gather_facts: no
  tasks:
    - name: create security group with cidr ethertype
      waldur_os_security_group:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: waldur-dev-infra
        description: some descr
        rules:
        - from_port: 80
          to_port: 80
          cidr: 2002::/16
          protocol: tcp
          ethertype: IPv6
        state: present
        name: sec-group-with-ethertype

- name: add security group with ingress direction
  hosts: localhost
  gather_facts: no
  tasks:
    - name: create security group with direction
      waldur_os_security_group:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: waldur-dev-infra
        description: some descr
        rules:
        - from_port: 80
          to_port: 80
          cidr: 0.0.0.0/00
          protocol: tcp
          direction: ingress
        state: present
        name: sec-group-with-direction

- name: add security group with egress direction
  hosts: localhost
  gather_facts: no
  tasks:
    - name: create security group with direction
      waldur_os_security_group:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: waldur-dev-infra
        description: some descr
        rules:
        - from_port: 80
          to_port: 80
          cidr: 0.0.0.0/00
          protocol: tcp
          direction: egress
        state: present
        name: sec-group-with-direction
"""


def compare_rules(local_rules, remote_rules):
    remote_rules = remote_rules.copy()

    for local_rule in local_rules:
        try:
            tmp_remote_rules = copy.deepcopy(remote_rules)

            for rules in tmp_remote_rules:
                if "remote_group" in local_rule:
                    if "cidr" in rules:
                        rules.pop("cidr")
                    if "ethertype" in rules:
                        rules.pop("ethertype")
                else:
                    if "remote_group" in rules:
                        rules.pop("remote_group")

            remote_rules.pop(tmp_remote_rules.index(local_rule))
        except ValueError:
            return False

    if not len(remote_rules):
        return True

    return False


def compare_description(description_1, description_2):
    description_1 = (
        description_1.strip() if isinstance(description_1, str) else description_1
    )
    description_2 = (
        description_2.strip() if isinstance(description_2, str) else description_2
    )

    if description_1 == description_2:
        return True

    if not bool(description_1) and not bool(description_2):
        return True

    return False


def get_tenant_uuid(client, waldur_resource_uuid):
    response = client.get_marketplace_resource(waldur_resource_uuid)
    scope_url = response["scope"]
    response = client._get(scope_url, valid_states=[200])
    return response["uuid"]


def send_request_to_waldur(client, module):
    has_changed = False
    tenant = module.params.get("tenant")
    waldur_resource = module.params.get("waldur_resource")

    if not tenant and not waldur_resource:
        raise AnsibleError("Tenant or waldur_resource must be specified.")

    if not tenant:
        tenant = get_tenant_uuid(client, waldur_resource)

    project = module.params.get("project")
    name = module.params["name"]
    description = module.params.get("description") or ""
    rules = module.params["rules"]

    for rule in rules:
        for item in ["from_port", "to_port", "protocol"]:
            if item not in rule:
                module.fail_json(msg="A rule must contain %s parameter." % item)

        if "cidr" in rule and "remote_group" in rule:
            module.fail_json(
                msg="Either cidr or remote_group must be specified, not both."
            )

        if "remote_group" in rule:
            remote_group = client.get_security_group(tenant, rule["remote_group"])
            rule["remote_group"] = remote_group["url"]
        elif "cidr" in rule:
            address = rule["cidr"]
            if "ethertype" not in rule or rule["ethertype"] == "IPv4":
                try:
                    IPv4Interface(address)
                except (AddressValueError, NetmaskValueError) as e:
                    module.fail_json(msg="Invalid IPv4 address %s: %s" % (address, e))
                else:
                    rule["ethertype"] = "IPv4"
            elif rule["ethertype"] == "IPv6":
                try:
                    IPv6Interface(address)
                except (AddressValueError, NetmaskValueError) as e:
                    module.fail_json(msg="Invalid IPv6 address %s: %s" % (address, e))
            else:
                module.fail_json(msg="Invalid ethertype: %s" % rule["ethertype"])
        else:
            module.fail_json(msg="Either cidr or remote_group must be specified.")

        if "direction" not in rule:
            rule["direction"] = "ingress"
        else:
            if rule["direction"] not in ["ingress", "egress"]:
                module.fail_json(
                    msg="Invalid direction %s expected ingress or egress"
                    % rule["direction"]
                )

    security_group = client.get_security_group(tenant, name)
    present = module.params["state"] == "present"

    if security_group:
        if present:
            rules_comp = [
                {
                    k: v
                    for k, v in rule.items()
                    if k
                    in [
                        "from_port",
                        "to_port",
                        "cidr",
                        "protocol",
                        "direction",
                        "description",
                        "ethertype",
                        "remote_group",
                    ]
                }
                for rule in security_group["rules"]
            ]
            if compare_description(
                security_group["description"], description
            ) and compare_rules(rules, rules_comp):
                has_changed = False
            else:
                if not compare_description(security_group["description"], description):
                    client.update_security_group_description(
                        security_group, description
                    )
                    has_changed = True

                if not compare_rules(rules, rules_comp):
                    client.update_security_group_rules(security_group, rules)
                    has_changed = True
        else:
            client.delete_security_group(security_group["uuid"])
            has_changed = True
    elif present:
        client.create_security_group(
            project=project,
            tenant=tenant,
            name=name,
            description=description,
            rules=rules,
            tags=module.params.get("tags"),
            wait=module.params["wait"],
            interval=module.params["interval"],
            timeout=module.params["timeout"],
        )
        has_changed = True

    return has_changed


def main():
    fields = waldur_resource_argument_spec(
        rules=dict(type="list", required=False, default=[]),
        project=dict(type="str", required=False),
        tenant=dict(type="str", required=False),
        waldur_resource=dict(type="str", required=False),
    )
    module = AnsibleModule(
        argument_spec=fields,
    )

    client = waldur_client_from_module(module)

    try:
        has_changed = send_request_to_waldur(client, module)
    except WaldurClientException as e:
        module.fail_json(msg=str(e))
    else:
        module.exit_json(changed=has_changed)


if __name__ == "__main__":
    main()



#!/usr/bin/python

from ansible.module_utils.basic import AnsibleModule
from waldur_client import (
    WaldurClientException,
    waldur_client_from_module,
    waldur_full_argument_spec,
)

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "OpenNode",
}

DOCUMENTATION = """
---
module: waldur_batch_allocation
short_description: Creates an order for batch allocation
options:
  access_token:
    description:
      - An access token which has permissions to manage an allocation.
    required: true
  api_url:
    description:
      - Fully qualified URL to the api endpoint.
    required: true
  project:
    description:
      - The name or UUID of the project to add an allocation to.
    required: true
  offering:
    description:
      - The name or UUID of the marketplace offering.
    required: true
  plan:
    description:
      - The name or UUID of the marketplace plan.
    required: true
  name:
    description:
      - The name of allocation
    required: true
  description:
    description:
      - The description of allocation
    required: true
"""

EXAMPLES = """
---
- hosts: localhost
  gather_facts: no
  tasks:
  - name: Create sample batch allocation
    waldur_batch_allocation:
      access_token: 5046870cf37bfe3f347d5cbcebed48f752912c9a
      api_url: https://waldur.example.com:8000/api
      project: Project
      offering: Offering name
      plan: Plan name
      name: Sample name
      description: Sample description
"""


def format_params(params):
    project = params["project"]
    offering = params["offering"]

    # 'module.params' contains each fields key
    # regardless if it is mentioned in a playbook or not

    plan = params["plan"]

    attributes = {"name": params["name"], "description": params["description"]}

    return project, offering, plan, attributes


def send_request_to_waldur(client, module):
    project, offering, plan, attributes = format_params(module.params)

    response = client.create_marketplace_order(project, offering, plan, attributes)
    return response, True


def main():
    fields = {
        "project": {"required": True, "type": "str"},
        "offering": {"required": True, "type": "str"},
        "plan": {"required": True, "type": "str"},
        "name": {"required": True, "type": "str"},
        "description": {"required": True, "type": "str"},
    }
    module = AnsibleModule(argument_spec=waldur_full_argument_spec(**fields))

    client = waldur_client_from_module(module)

    try:
        order, has_changed = send_request_to_waldur(client, module)
    except WaldurClientException as e:
        module.fail_json(msg=str(e))
    else:
        module.exit_json(order=order, changed=has_changed)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# has to be a full import due to Ansible 2.0 compatibility
from ansible.module_utils.basic import AnsibleModule
from waldur_client import WaldurClientException, waldur_client_from_module

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "OpenNode",
}

DOCUMENTATION = """
---
module: waldur_os_security_group_gather_facts
short_description: Get OpenStack tenant security group
version_added: 0.1
description:
  - "Get OpenStack tenant security group"
requirements:
  - "python = 3.8"
  - "requests"
  - "python-waldur-client"
options:
  access_token:
    description:
      - An access token which has permission to read a security group.
    required: true
  api_url:
    description:
      - Fully qualified URL to the Waldur.
    required: true
  name:
    description:
      - The name of the security group.
    required: false
  tenant:
    description:
      - The name of the tenant.
    required: true
"""

EXAMPLES = """
- name: get security group
  hosts: localhost
  tasks:
    - name: get security group
      waldur_os_security_group_gather_facts:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: VPC #1
        name: classic-web

- name: list tenant security groups
  hosts: localhost
  tasks:
    - name: list all security groups belonging to the tenant
      waldur_os_security_group_gather_facts:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        tenant: VPC #1
"""


def send_request_to_waldur(client, module):
    tenant = module.params["tenant"]
    name = module.params["name"]
    if name:
        return [client.get_security_group(tenant, name)]
    else:
        return client.list_security_group(tenant)


def main():
    fields = dict(
        api_url=dict(required=True, type="str"),
        access_token=dict(required=True, type="str", no_log=True),
        name=dict(type="str", required=False),
        tenant=dict(type="str", required=True),
    )
    module = AnsibleModule(argument_spec=fields)

    client = waldur_client_from_module(module)

    try:
        security_groups = send_request_to_waldur(client, module)
    except WaldurClientException as e:
        module.fail_json(msg=str(e))
    else:
        module.exit_json(security_groups=security_groups)


if __name__ == "__main__":
    main()



import unittest
from unittest import mock

import waldur_os_instance_volume


class BaseVolumeTest(unittest.TestCase):
    def setUp(self):
        self.module = mock.Mock()
        self.module.params = {
            "project": "database management",
            "volume": "postgresql-data",
            "instance": "postgresql-server",
            "device": "/dev/vdb",
            "state": "absent",
            "wait": False,
            "interval": 10,
            "timeout": 600,
        }


class VolumeDetachTest(BaseVolumeTest):
    def test_volume_is_already_detached(self):
        client = mock.Mock()
        client.get_volume.return_value = {
            "runtime_state": "available",
        }
        has_changed = waldur_os_instance_volume.send_request_to_waldur(
            client, self.module
        )
        self.assertFalse(has_changed)

    def test_volume_is_attached(self):
        client = mock.Mock()
        client.get_volume.return_value = {
            "runtime_state": "in-use",
            "uuid": "volume_uuid",
        }
        has_changed = waldur_os_instance_volume.send_request_to_waldur(
            client, self.module
        )
        self.assertTrue(has_changed)
        client.detach_volume.assert_called_once()


class VolumeAttachTest(BaseVolumeTest):
    def setUp(self):
        super(VolumeAttachTest, self).setUp()
        self.module.params["state"] = "present"

    def test_volume_is_already_attached(self):
        client = mock.Mock()
        client.get_volume.return_value = {
            "runtime_state": "in-use",
            "instance": "instance_url",
        }
        client.get_instance.return_value = {
            "url": "instance_url",
        }
        has_changed = waldur_os_instance_volume.send_request_to_waldur(
            client, self.module
        )
        self.assertFalse(has_changed)

    def test_volume_is_attached_to_another_instance(self):
        client = mock.Mock()
        client.get_volume.return_value = {
            "runtime_state": "in-use",
            "uuid": "volume_uuid",
            "instance": "instance_url",
        }
        client.get_instance.return_value = {
            "url": "another_instance_url",
            "uuid": "another_instance_uuid",
        }
        has_changed = waldur_os_instance_volume.send_request_to_waldur(
            client, self.module
        )
        self.assertTrue(has_changed)
        client.detach_volume.assert_called_once_with("volume_uuid")
        client.attach_volume.assert_called_once_with(
            "volume_uuid",
            "another_instance_uuid",
            "/dev/vdb",
            interval=10,
            timeout=600,
            wait=False,
        )



#!/usr/bin/python

from ansible.module_utils.basic import AnsibleModule
from waldur_client import (
    WaldurClientException,
    waldur_client_from_module,
    waldur_full_argument_spec,
)

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "OpenNode",
}

DOCUMENTATION = """
---
options:
    api_url:
        description:
            - Fully qualified URL to the Waldur.
        required: true
    access_token:
        description:
            - An access token which has permissions to create offerings.
        required: true
    name:
        description:
            - The name of the new batch offering.
        required: true
    description:
        description:
            - The description of the new batch offering.
    full_description:
        description:
            - The full description of the new batch offering.
    native_name:
        description:
            - The native name of the new batch offering.
    native_description:
        description:
            - The native description of the new batch offering.
    terms_of_service:
        description:
            - The terms of the service.
    provider:
        description:
            - The name or UUID of the provider organization.
    rating:
        description:
            - The rating form 1 to 5.
    category:
        description:
            - The name or UUID of the marketplace category.
        required: true
    attributes:
        description:
            - The attributes of the marketplace category.
    geolocations:
        description:
            - The geolocations of provided service.
    plans:
        description:
            - The list of plans attached to offering.
        required: true
    batch_service:
        description:
            - The batch service of allocation (MOAB or SLURM).
        required: true
    hostname:
        description:
            - Hostname or IP address of master node.
        required: true
    username:
        description:
            - Username for SSH connection.
        required: true
    port:
        description:
            - Port of the master node process.
    gateway:
        description:
            - Hostname or IP address of gateway node.
    use_sudo:
        description:
            - The flag for privilege escalation activation.
    default_account:
        description:
            - Default SLURM account for user.
        required: true
    shared:
        description:
            - The flag of a possible access to all organizations.
    billable:
        description:
            - The flag if purchase and usage is invoiced.
    datacite_doi:
        description:
            - Persistent ID for the service.
"""

EXAMPLES = """
---
- hosts: localhost
  gather_facts: no
  tasks:
    - name: Create HPC allocation
      check_mode: no
      waldur_batch_offering:
        access_token: 3cecd050b7c1dbce54bc45bf508ec836aecdd5bc
        api_url: http://193.40.155.148:8000/api
        name: Offering sample name
        native_name:  Offering sample native name
        description: Offering description
        native_description: Offering native description
        full_description: Offering full description
        terms_of_service: Sample terms
        category: dfe535350663441e99615b0be014c742
        provider: cba70dd7f75d40dc9650704198e9071a
        batch_service: SLURM
        hostname: localhost
        port: 8080
        username: user
        default_account: root
        gateway: localhost
        shared: true
        plans:
        - name: HPC plan
          unit: month
          prices:
            cpu: 100
            gpu: 70
            ram: 64
        geolocations:
        - latitude: 59.3990796
          longitude: 26.6625565
"""


def format_params(params):
    excluded_keys = [
        "batch_service",
        "hostname",
        "username",
        "default_account",
        "port",
        "gateway",
        "type",
        "provider",
    ]

    formatted_params = {
        k: v for (k, v) in params.items() if v and k not in excluded_keys
    }

    formatted_params["type"] = "SlurmInvoices.SlurmPackage"

    if params.get("provider"):
        formatted_params["customer"] = params["provider"]

    service_attributes = {
        "batch_service": params["batch_service"],
        "hostname": params["hostname"],
        "username": params["username"],
        "default_account": params["default_account"],
    }
    if params.get("port"):
        service_attributes["port"] = params["port"]
    if params.get("gateway"):
        service_attributes["gateway"] = params["gateway"]
    formatted_params["service_attributes"] = service_attributes

    return formatted_params


def send_request_to_waldur(client, module):
    params = format_params(module.params)
    offering, changed = client.create_offering(params, module.check_mode)
    return offering, changed


def main():
    fields = {
        # Overview
        "name": {"required": True, "type": "str"},
        "description": {"required": False, "type": "str"},
        "full_description": {"required": False, "type": "str"},
        "native_name": {"required": False, "type": "str"},
        "native_description": {"required": False, "type": "str"},
        "terms_of_service": {"required": False, "type": "str"},
        # Organization details
        "provider": {"required": False, "type": "str"},
        "rating": {"required": False, "type": "int", "choices": list(range(1, 6))},
        # Description
        "category": {"required": True, "type": "str"},
        "attributes": {"required": False, "type": "dict"},  # category attributes
        "geolocations": {"required": False, "type": "list"},
        # Accounting
        "plans": {"required": True, "type": "list"},
        # Management
        "batch_service": {
            "required": True,
            "type": "str",
            "choices": ["SLURM", "MOAB"],
        },
        "hostname": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "port": {"required": False, "type": "int"},
        "gateway": {"required": False, "type": "str"},
        "use_sudo": {"required": False, "type": "str"},
        "default_account": {"required": True, "type": "str"},
        "shared": {"required": False, "type": "bool"},
        "billable": {"required": False, "type": "bool"},
        "datacite_doi": {"required": False, "type": "str"},
    }

    module = AnsibleModule(
        argument_spec=waldur_full_argument_spec(**fields), supports_check_mode=True
    )

    client = waldur_client_from_module(module)

    try:
        offering, changed = send_request_to_waldur(client, module)

        module.exit_json(offering=offering, changed=changed)
    except WaldurClientException as e:
        module.fail_json(msg=str(e))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# has to be a full import due to Ansible 2.0 compatibility

from ansible.module_utils.basic import AnsibleModule
from waldur_client import (
    WaldurClient,
    WaldurClientException,
    waldur_client_from_module,
    waldur_resource_argument_spec,
)

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "OpenNode",
}

DOCUMENTATION = """
---
module: waldur_os_subnet
short_description: The creation and management of OpenStack subnets
version_added: 0.1
description:
  - "Manage OpenStack subnets"
requirements:
  - "python = 3.8"
  - "requests"
  - "python-waldur-client"
options:
  access_token:
    description:
      - An access token which has permissions to create and modify a subnet.
    required: true
  api_url:
    description:
      - Fully qualified URL to the Waldur.
    required: true
  uuid:
    description:
      - Unique identifier for subnet
  tenant:
    description:
      - The name or uuid of the tenant that the subnet should be connected to
    required: true
  project:
    description:
      - The name or uuid of the project that the subnet is apart of
  network_uuid:
    description:
      - Unique identifier of the network to associate with the subnet
  cidr:
    description:
      - The CIDR notation for the subnet
  gateway_ip
    description:
      - Specific IP address to use as the gateway
  disable_gateway:
    description:
      - Disable the gateway for a subnet
  allocation_pools
    description:
      - Allocation pool IP addresses for the subnet
  enable_dhcp
    description:
      - Enable dhcp on the subnet
  dns_nameservers
    description:
      - DNS nameservers for the subnet
  connect_subnet
    description:
    - Connect subnet
  disconnect_subnet
    description:
    - Disconnect subnet
  unlink_subnet:
    description:
    - Delete subnet from the database without scheduling operations on backend.
  state
    choices:
    - present
    - absent
    description:
      - Should the resource be present or absent

"""
EXAMPLES = """
- name: Connect a subnet to a router in a network
  hosts: localhost
  tasks:
    - name: connect subnet
      waldur_os_subnet:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        uuid: 935c8841fd1644228a9d1463e8693650
        project: feline-suppliment-sourcer
        connect_subnet: True

- name: Disconnect a subnet from the router
  hosts: localhost
  tasks:
    - name: disconnect subnet
      waldur_os_subnet:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        uuid: 935c8841fd1644228a9d1463e8693650
        project: feline-suppliment-sourcer
        disconnect_subnet: True

- name: Unlink a subnet
  hosts: localhost
  tasks:
    - name: destroy subnet
      waldur_os_subnet:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        uuid: 935c8841fd1644228a9d1463e8693650
        project: feline-suppliment-sourcer
        unlink_subnet: True

- name: Update a subnet
  hosts: localhost
  tasks:
    - name: update subnet
      waldur_os_subnet:
        access_token: b83557fd8e2066e98f27dee8f3b3433cdc4183ce
        api_url: https://waldur.example.com:8000/api
        uuid: 935c8841fd1644228a9d1463e8693650
        name: vanessa-nutrition-subnet
        project: feline-suppliment-sourcer
        gateway_ip: 192.168.42.2

"""


def compare_fields(checked_fields, local_fields):
    try:
        field_diff = {
            k: checked_fields[k]
            for k, v in checked_fields
            if k in local_fields
            if local_fields[k, v] != checked_fields[k, v]
        }
        if field_diff:
            return True
        else:
            return False
    except ValueError:
        return False


def send_request_to_waldur(client: WaldurClient, module):
    has_changed = False
    subnet_uuid = module.params.get("subnet_uuid")
    name = module.params.get("name")
    tenant = module.params.get("tenant")
    project = module.params.get("project")
    network_uuid = module.params.get("network_uuid")
    cidr = module.params.get("cidr")
    gateway_ip = module.params.get("gateway_ip")
    disable_gateway = module.params.get("disable_gateway")
    allocation_pools = module.params.get("allocation_pools")
    enable_dhcp = module.params.get("enable_dhcp")
    dns_nameservers = module.params.get("dns_nameservers")
    connect_subnet = module.params.get("connect_subnet")
    disconnect_subnet = module.params.get("disconnect_subnet")
    unlink_subnet = module.params.get("unlink_subnet")
    state = module.params.get("state")

    subnet = None
    present = state == "present"

    subnet = client.get_subnet_by_uuid(subnet_uuid)
    if subnet:
        if present:
            checked_fields = {
                k: v
                for k, v in subnet.items()
                if k
                in [
                    "name",
                    "tenant",
                    "gateway_ip",
                    "disable_gateway",
                    "enable_dhcp",
                    "dns_nameservers",
                    "connect_subnet",
                    "disconnect_subnet",
                    "unlink_subnet",
                ]
            }
            local_fields = [
                name,
                tenant,
                gateway_ip,
                disable_gateway,
                enable_dhcp,
                dns_nameservers,
                connect_subnet,
                disconnect_subnet,
                unlink_subnet,
            ]

            if compare_fields(checked_fields, local_fields):
                has_changed = False
            else:
                if not compare_fields(checked_fields, local_fields):
                    client.update_subnet(
                        uuid=subnet_uuid,
                        name=name,
                        tenant=tenant,
                        gateway_ip=gateway_ip,
                        disable_gateway=disable_gateway,
                        enable_dhcp=enable_dhcp,
                        dns_nameservers=dns_nameservers,
                        connect_subnet=connect_subnet,
                        disconnect_subnet=disconnect_subnet,
                        unlink_subnet=unlink_subnet,
                    )
                    has_changed = True

    else:
        if present:
            subnet = client.create_subnet(
                name=name,
                tenant=tenant,
                project=project,
                network_uuid=network_uuid,
                cidr=cidr,
                allocation_pools=allocation_pools,
                enable_dhcp=enable_dhcp,
                dns_nameservers=dns_nameservers,
                disable_gateway=disable_gateway,
                gateway_ip=gateway_ip,
                wait=module.params["wait"],
                interval=module.params["interval"],
                timeout=module.params["timeout"],
            )
        has_changed = True

    return subnet, has_changed


def main():
    fields = waldur_resource_argument_spec(
        subnet_uuid=dict(type="str"),
        name=dict(type="str", required=False),
        tenant=dict(type="str", required=False),
        project=dict(type="str", required=False),
        network_uuid=dict(type="str", required=False),
        cidr=dict(type="str", required=False),
        allocation_pools=dict(type="str", required=False),
        enable_dhcp=dict(type="bool", required=False, default=True),
        dns_nameservers=dict(type="list", required=False),
        disable_gateway=dict(type="str", required=False),
        gateway_ip=dict(type="str", required=False),
        connect_subnet=dict(type="bool", required=False),
        disconnect_subnet=dict(type="bool", required=False),
        unlink_subnet=dict(type="bool", required=False),
    )
    module = AnsibleModule(
        argument_spec=fields,
    )

    client = waldur_client_from_module(module)

    gateway_ip = module.params.get("gateway_ip")
    disable_gateway = module.params.get("gateway_ip")

    try:
        has_changed = send_request_to_waldur(client, module)
    except WaldurClientException as e:
        module.fail_json(msg=str(e))
    if gateway_ip:
        if disable_gateway is True:
            module.fail_json(
                msg="Gateway IP cannot be set while disable_gateway is True"
            )
    if disable_gateway is False:
        if not gateway_ip:
            module.fail_json(msg="Gateway ip must be set or gateway must be disabled")
    else:
        module.exit_json(changed=has_changed)


if __name__ == "__main__":
    main()



import unittest
from unittest import mock

import waldur_marketplace_os_instance


class InstanceSubNetUpdateTest(unittest.TestCase):
    def setUp(self):
        module = mock.Mock()
        self.subnets_set = ["subnet_1", "subnet_2"]
        module.params = {
            "name": "Test instance",
            "project": "Test project",
            "subnet": self.subnets_set,
            "state": "present",
            "wait": True,
            "interval": 20,
            "timeout": 600,
        }
        module.check_mode = False
        self.module = module

    def test_connect_marketplace_instance_to_multiple_subnets_using_network_syntax(
        self,
    ):
        client = mock.Mock()
        instance_uuid = "59e46d029a79473779915a22"
        client.get_instance_via_marketplace.return_value = {
            "uuid": instance_uuid,
            "ports": [
                {
                    "subnet_name": "Test subnet",
                    "subnet_uuid": "77e46d029a79473779915a22",
                }
            ],
            "security_groups": [],
        }

        module = mock.Mock()
        module.params = {
            "name": "Test instance",
            "project": "Test project",
            "networks": [
                {"subnet": subnet_name, "floating_ip": "auto"}
                for subnet_name in self.subnets_set
            ],
            "state": "present",
            "wait": True,
            "interval": 20,
            "timeout": 600,
        }
        module.check_mode = False

        _, has_changed = waldur_marketplace_os_instance.send_request_to_waldur(
            client, module
        )
        client.update_instance_ports.assert_called_once_with(
            instance_uuid=instance_uuid,
            subnet_set=self.subnets_set,
            interval=20,
            timeout=600,
            wait=True,
        )
        self.assertTrue(has_changed)

    def test_update_ip_of_marketplace_instance(self):
        client = mock.Mock()
        instance_uuid = "59e46d029a79473779915a22"
        client.get_instance_via_marketplace.return_value = {
            "uuid": instance_uuid,
            "ports": [],
            "security_groups": [],
        }

        module = mock.Mock()
        module.params = {
            "name": "Test instance",
            "project": "Test project",
            "subnet": self.subnets_set[0],
            "floating_ip": "auto",
            "state": "present",
            "wait": True,
            "interval": 20,
            "timeout": 600,
        }
        module.check_mode = False

        _, has_changed = waldur_marketplace_os_instance.send_request_to_waldur(
            client, module
        )
        client.update_instance_ports.assert_called_once_with(
            instance_uuid=instance_uuid,
            subnet_set=[self.subnets_set[0]],
            interval=20,
            timeout=600,
            wait=True,
        )
        self.assertTrue(has_changed)



#!/usr/bin/env python

from setuptools import setup

install_requires = [
    "requests>=2.6.0",
    "python-waldur-client>=0.4.3",
    "urllib3<2.0.0",
]

tests_requires = [
    "responses>=0.5.0",
]


setup(
    name="ansible-waldur-module",
    version="1.2.0",
    author="OpenNode Team",
    author_email="info@opennodecloud.com",
    url="https://waldur.com",
    license="MIT",
    description="Ansible module for the Waldur API.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    py_modules=[
        "waldur_client",
        "waldur_marketplace",
        "waldur_marketplace_os_get_instance",
        "waldur_marketplace_os_instance",
        "waldur_marketplace_os_volume",
        "waldur_os_floating_ip",
        "waldur_os_instance_volume",
        "waldur_os_security_group",
        "waldur_os_security_group_gather_facts",
        "waldur_os_snapshot",
        "waldur_batch_allocation",
        "waldur_batch_offering",
        "waldur_os_subnet",
        "waldur_os_subnet_gather_facts",
    ],
    install_requires=install_requires,
    tests_require=tests_requires,
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import unittest
from unittest import mock

import waldur_os_subnet


def fail_side_effect(*args, **kwargs):
    raise Exception(kwargs["msg"])


class BaseSubnetTest(unittest.TestCase):
    def setUp(self):
        self.module = mock.Mock()

        self.module.params = {
            "access_token": "token",
            "api_url": "api",
            "uuid": "df3ee5cac5874dffa1aad86bc1919d8d",
            "name": "subnet",
            "tenant": "tenant",
            "project": "Test-project",
            "dns_nameservers": ["8.8.8.8", "8.8.4.4"],
            "disable_gateway": False,
            "gateway_ip": "192.168.42.1",
            "state": "present",
            "wait": True,
            "interval": 10,
            "timeout": 600,
        }
        self.client = mock.Mock()

    # adapted from waldur_os_security_group
    def check_unsuccessful_function_call(self, msg):
        self.module.fail_json.side_effect = fail_side_effect

        self.assertRaisesRegex(
            Exception,
            msg,
            waldur_os_subnet.send_request_to_waldur,
            self.client,
            self.module,
        )

    def test_valid_gateway_config(self):
        client = mock.Mock()
        client.get_subnet_by_uuid.return_value = {
            "uuid": "uuid",
            "disable_gateway": False,
            "gateway_ip": "192.168.42.2",
        }
        has_changed = waldur_os_subnet.send_request_to_waldur(client, self.module)
        self.assertTrue(has_changed)

    def test_name_update(self):
        client = mock.Mock()
        self.module.params["name"] = "subnet_test"
        client.get_subnet_by_uuid.return_value = {
            "uuid": "uuid",
            "name": "subnet_test",
        }
        has_changed = waldur_os_subnet.send_request_to_waldur(client, self.module)
        self.assertTrue(has_changed)

    def test_create_subnet(self):
        client = mock.Mock()
        self.module.params = {
            "name": "subnet-creation-net",
            "project": "8bd6d84651354c298136b821b9b73626",
            "gateway_ip": "192.168.42.1",
            "disable_gateway": False,
            "cidr": "192.168.42.0/24",
            "dns_nameservers": ["8.8.8.8", "8.8.4.4"],
            "wait": True,
            "interval": 10,
            "timeout": 600,
            "network_uuid": "96172a4c5cf240539778f41ddffda145",
            "tenant": "adaa0a33d18845d7b3af388bfea8208c",
        }
        self.module.check_mode = False
        has_changed = waldur_os_subnet.send_request_to_waldur(client, self.module)
        self.assertTrue(has_changed)

        pass

    def test_dns_nameserver_update(self):
        client = mock.Mock()
        self.module.params["dns_nameservers"] = []
        client.get_subnet_by_uuid.return_value = {
            "uuid": "uuid",
            "dns_nameservers": [],
        }
        has_changed = waldur_os_subnet.send_request_to_waldur(client, self.module)
        self.assertTrue(has_changed)



import tempfile
import unittest
from unittest import mock

import waldur_marketplace


@mock.patch("waldur_marketplace.waldur_client_from_module")
@mock.patch("waldur_marketplace.AnsibleModule")
class OrderItemCreateTest(unittest.TestCase):
    def setUp(self):
        module = mock.Mock()
        module.params = {
            "api_url": "http://example.com:8000/api",
            "access_token": "token",
            "offering": "Test offering",
            "plan": "Test plan",
            "project": "Test project",
        }
        module.check_mode = False
        self.module = module

    def test_create_marketplace_order(self, mock_ansible_module, mock_waldur_client):
        client = mock.Mock()
        client.create_marketplace_order.return_value = {"uuid": "order_uuid"}
        _, has_changed = waldur_marketplace.send_request_to_waldur(client, self.module)
        client.create_marketplace_order.assert_called_once()
        self.assertTrue(has_changed)

    def test_fail_json_is_called_if_file_is_not_found(
        self, mock_ansible_module, mock_waldur_client
    ):
        self.module.params["attributes"] = "/file/not/found.json"
        mock_ansible_module.return_value = self.module
        waldur_marketplace.main()
        self.module.fail_json.assert_called_once_with(
            msg="Unable to open file: [Errno 2] No such file or directory: '/file/not/found.json'"
        )

    def test_exit_json_is_called_if_file_is_found(
        self, mock_ansible_module, mock_waldur_client
    ):
        tmp_file = tempfile.NamedTemporaryFile(delete=False, mode="w")
        tmp_file.write('{"name": "my name"}')
        tmp_file.close()
        self.module.params["attributes"] = tmp_file.name
        mock_ansible_module.return_value = self.module
        waldur_marketplace.main()
        self.module.fail_json.assert_not_called()
        self.module.exit_json.assert_called_once()
